/**
 * Program RobotInfo.java
 * 
 * Date         Version    Author         Comment
 * 2008/03/14   1.0.0      T.Kawashima    First.
 * 
 */

package robomin.robot;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.logging.Level;

import robomin.util.RobominDef;
import robomin.util.RobominInfo;
import robomin.util.RobominLogger;

/**
 * {bgۑNX
 * @author T.Kawashima
 * @version 1.0.0
 */
public class RobotInfo {
	private String name;
	private String ipAddr;
	private Socket socket = null;
	private BufferedReader sockin;
	private BufferedWriter sockout;
	private RobominLogger logger;
	private int portnum = 8081;
	
	/**
	 * RXgN^
	 * @param name O
	 * @param ipAddr IPAhX
	 */
	public RobotInfo(RobominInfo info, String name, String ipAddr) {
		this.logger = info.getRobominLogger();
		this.name = name;
		this.ipAddr = ipAddr;
	}
	
	/**
	 * RXgN^
	 * @param name O
	 * @param ipAddr IPAhX
	 * @param portnum |[gԍ
	 */
	public RobotInfo(RobominInfo info, String name, String ipAddr, int portnum) {
		this(info, name, ipAddr);
		this.portnum = portnum;
	}
	
	/**
	 * {bg̖OԂ
	 * @return O
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * {bgIPǂꂷԂ
	 * @return IPAhX
	 */
	public String getAddr() {
		return this.ipAddr;
	}
	
	/**
	 * {bgƐڑ
	 * @return 0 s<0
	 */
	public int connectRobot() {
		return this.openSocket(ipAddr, portnum);
	}
	
	/**
	 * {bgƐڑؒf
	 * @return 0 s<0
	 */
	public int disconnectRobot() {
		if (isConnected()) {
			return this.closeSocket();
		}
		
		return RobominDef.SUCCESS;
	}
	
	/**
	 * SocketI[v
	 * @param ipAddress ڑIPAhX
	 * @param portnum@ڑ|[gԍ
	 * @return 0 s<0
	 */
	private int openSocket(String ipAddress, int portnum) {
		try {
			socket = new Socket(ipAddress, portnum);
		    sockin = new BufferedReader(new InputStreamReader(socket.getInputStream()));
		    sockout = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
		}
		catch (IOException e) {
			logger.log(Level.SEVERE, "Exception", e);
			return RobominDef.FAIL_CONNECT;
		}

		return 0;
	}
	
	/**
	 * SocketN[Y
	 * @return ꍇ0 sꍇ-1
	 */
	private int closeSocket() {
		int res = 0;
		
		try {
			socket.close();
			sockin.close();
			sockout.close();
		}
		catch (IOException e) {
			logger.log(Level.SEVERE, "Exception", e);
			res = RobominDef.FAIL;
		}
		
		return res;
	}
	
	/**
	 * BufferedWriterԂ
	 * @return BufferedWriter
	 */
	public BufferedWriter getBufferedWriter() {
		return this.sockout;
	}
	
	/**
	 * \Pbg̃RlNVԂ
	 * @return ڑTRUEAڑFALSE
	 */
	public boolean isConnected() {
		if (socket != null) {
			return socket.isConnected();
		}
		else {
			return false;
		}
	}
	
	public boolean isClosed() {
		if (socket != null) {
			return socket.isClosed();
		}
		else {
			return true;
		}
	}
}
