/**
 * Program NetTansorTab.java
 * 
 * Date         Version    Author         Comment
 * 2008/03/17   1.0.0      T.Kawashima    First.
 * 
 */

package robomin.ui;

import java.util.logging.Level;

import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import com.phidgets.event.AttachEvent;
import com.phidgets.event.AttachListener;
import com.phidgets.event.DetachEvent;
import com.phidgets.event.DetachListener;
import com.phidgets.event.ErrorEvent;
import com.phidgets.event.ErrorListener;
import com.phidgets.event.InputChangeEvent;
import com.phidgets.event.InputChangeListener;
import com.phidgets.event.OutputChangeEvent;
import com.phidgets.event.OutputChangeListener;
import com.phidgets.event.SensorChangeEvent;
import com.phidgets.event.SensorChangeListener;

import robomin.robot.NetTansorCmd;
import robomin.robot.RobotInfo;
import robomin.sensor.PhidgetDef;
import robomin.sensor.PhidgetIKP;
import robomin.sensor.PhidgetInfo;
import robomin.util.RobominDef;
import robomin.util.RobominInfo;
import robomin.util.RobominLogger;

/**
 * NetTansorp^u
 * @author T.Kawashima
 * @version 1.0.0
 */
public class NetTansorTab implements AttachListener, DetachListener, ErrorListener, InputChangeListener, OutputChangeListener, SensorChangeListener {
	private static final int PhidgetTimeout = 1000;
	
	private CTabItem tabItem;
	private int language = -1;
	private RobotInfo rInfo;
	private RobominLogger logger;
	private NetTansorCmd ntCmd;
	private PhidgetIKP pIKP;
	private PhidgetInfo pInfo;
	
	private Image tabIcon;
	
	// C^u
	private SashForm mainSash;
	private SashForm subSash;
	private SashForm subSash2;
	
	private Group opeGroup;

	// OiރR}hpl
	private Composite opeComp1;
	private Combo opeCombo1;
	private Spinner opeSp11;
	private Spinner opeSp12;
	private Composite opeComp11;
	private Button  opeBtn1;
	
	// ]EJ[uR}hpl
	private Composite opeComp2;
	private Combo opeCombo2;
	private Spinner opeSp21;
	private Spinner opeSp22;
	private Composite opeComp21;
	private Button opeBtn2;
	
	// ~R}hpl
	private Composite opeComp3;
	private Combo opeCombo3;
	private Composite opeComp31;
	private Button opeBtn3;
	
	// ߃R}h
	private Composite opeComp4;
	private Combo opeCombo4;
	private Spinner opeSp41;
	private Spinner opeSp42;
	private Spinner opeSp43;
	private Button opeBtn4;
	
	// sR}h
	private Composite opeComp5;
	private Combo opeCombo5;
	private Spinner opeSp51;
	private Spinner opeSp52;
	private Composite opeComp51;
	private Button opeBtn5;
	
	// [^[R}h
	private Composite opeComp6;
	private Combo opeCombo6;
	private Spinner opeSp61;
	private Spinner opeSp62;
	private Composite opeComp61;
	private Button opeBtn6;
	
	// Route Indicator
	private Group routeGroup;
	private Composite routeComp;
	private Button routeStartBtn;
	private Button routeStopBtn;
	private Button routeResetBtn;
	private Canvas routeCanvas;
	
	// Phidget Sensor
	private Group phidgetGroup;
	private Image phidgetNewConIcon;
	private Button phidgetNewConBtn;
	private Image phidgetReConIcon;
	private Button phidgetReConBtn;
	private Image phidgetDisBtnIcon;
	private Button phidgetDisBtn;
	private Composite phidgetComp1;
	private Composite phidgetComp2;
	
	private Font infoFont;
	
	private Group phidgetJoyGroup;
	private Label phidgetJoyLabel;
	private int phidgetJoyX;
	private int phidgetJoyY;
	
	private Group phidgetSliderGroup;
	private Label phidgetSliderLabel;
	private int phidgetSliderV;
	
	// Robomin Prompt
	private Group promptGroup;
	private Text promptText;
	final Point prePoint;
	
	/**
	 * RXgN^
	 * @param tabFolder ẽ^utH_
	 * @param info RobominInfo
	 */
	public NetTansorTab(CTabFolder tabFolder, final RobominInfo info) {
		language = info.getLanguage();
		logger = info.getRobominLogger();
		ImageRegistry imgReg = new ImageRegistry();
		
		// ACR̃[h
		try {
			tabIcon = new Image(Display.getDefault(), GUIDef.APPLICATION_ICONPATH);
			phidgetNewConIcon = new Image(Display.getDefault(), GUIDef.NTTAB_PHIDGET_NEWCONICON);
			phidgetReConIcon = new Image(Display.getDefault(), GUIDef.NTTAB_PHIDGET_RECONICON);
			phidgetDisBtnIcon = new Image(Display.getDefault(), GUIDef.NTTAB_PHIDGET_DISCONICON);
			imgReg.put(GUIDef.APPLICATION_ICONPATH, tabIcon);
			imgReg.put(GUIDef.NTTAB_PHIDGET_NEWCONICON, phidgetNewConIcon);
			imgReg.put(GUIDef.NTTAB_PHIDGET_RECONICON, phidgetReConIcon);
			imgReg.put(GUIDef.NTTAB_PHIDGET_DISCONICON, phidgetDisBtnIcon);
		}
		catch (SWTException e) {
			logger.log(Level.SEVERE, "G[", e);
		}
		
		pIKP = new PhidgetIKP(info);
		
		// CxgXi[̓o^
		pIKP.addAttachListener(this);
		pIKP.addDetachListener(this);
		pIKP.addErrorListener(this);
		pIKP.addInputChangeListener(this);
		pIKP.addOutputChangeListener(this);
		pIKP.addSensorChangeListener(this);
		
		tabItem = new CTabItem(tabFolder, SWT.NONE);
		tabItem.setText(GUIDef.TABFOLDER_NETTANSOR[language]);
		tabItem.setImage(tabIcon);
		tabFolder.setSelection(tabItem);
		
		mainSash = new SashForm(tabFolder, SWT.VERTICAL);
		subSash = new SashForm(mainSash, SWT.HORIZONTAL);
		subSash2 = new SashForm(mainSash, SWT.HORIZONTAL);
		
		// Iy[Vpl
		opeGroup = new Group(subSash, SWT.SHADOW_IN);
		opeGroup.setText(GUIDef.NTTAB_OPEGROUP[language]);
		opeGroup.setLayout(new FillLayout(SWT.VERTICAL));
		opeGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		// OiEރR}hMpl
		Group opeGroup1 = new Group(opeGroup, SWT.NONE);
		opeGroup1.setLayout(new GridLayout(1, true));
		opeGroup1.setLayout(new FillLayout());
		opeComp1 = new Composite(opeGroup1, SWT.NONE);
		opeComp1.setLayout(new RowLayout(SWT.HORIZONTAL));
		opeCombo1 = new Combo(opeComp1, SWT.NONE);
		opeCombo1.setLayoutData(new RowData(GUIDef.NTTAB_OPE_COMBO_X, GUIDef.NTTAB_OPE_COMBO_Y));
		opeCombo1.add(GUIDef.NTTAB_OPE_FWCMD[language]);
		opeCombo1.add(GUIDef.NTTAB_OPE_BKCMD[language]);
		opeCombo1.setText(GUIDef.NTTAB_OPE_FWCMD[language]);
		opeCombo1.select(0);
		opeSp11 = new Spinner(opeComp1, SWT.BORDER);
		opeSp11.setMinimum(RobominDef.NTPOWER_MIN);
		opeSp11.setMaximum(RobominDef.NTPOWER_MAX);
		opeSp11.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp11.setToolTipText(GUIDef.NTTAB_OPESP11_TOOLTIP[language]);
		opeSp12 = new Spinner(opeComp1, SWT.BORDER);
		opeSp12.setMinimum(RobominDef.NTDISTANCE_MIN);
		opeSp12.setMaximum(RobominDef.NTDISTANCE_MAX);
		opeSp12.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp12.setToolTipText(GUIDef.NTTAB_OPESP12_TOOLTIP[language]);
		opeComp11 = new Composite(opeComp1, SWT.NONE);
		opeComp11.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPACE_X-1, GUIDef.NTTAB_OPE_SPACE_Y));
		opeBtn1 = new Button(opeComp1, SWT.NONE);
		opeBtn1.setText(GUIDef.NTTAB_OPE_SENDBTN[language]);
		opeBtn1.setLayoutData(new RowData(GUIDef.NTTAB_OPE_BTN_X, GUIDef.NTTAB_OPE_BTN_Y));
		opeBtn1.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "FB Command");

				if (rInfo != null) {
					int res = 0;

					if (opeCombo1.getSelectionIndex() == 0) {
						res = ntCmd.sendCmdForward(rInfo, opeSp11.getSelection(), opeSp12.getSelection(), -1, -1, -1);
					}
					else if (opeCombo1.getSelectionIndex() == 1) {
						res = ntCmd.sendCmdBackward(rInfo, opeSp11.getSelection(), opeSp12.getSelection(), -1, -1, -1);
					}
					
					if (res < 0) {
						logger.log(Level.WARNING, "Send FB command error ErrorCode(" + res + ")");
					}
				}
			}
		});
		
		// E]E]EEJ[uEJ[uR}hMpl
		Group opeGroup2 = new Group(opeGroup, SWT.NONE);
		opeGroup2.setLayout(new FillLayout());
		opeComp2 = new Composite(opeGroup2, SWT.NONE);
		opeComp2.setLayout(new RowLayout(SWT.HORIZONTAL));
		opeCombo2 = new Combo(opeComp2, SWT.NONE);
		opeCombo2.setLayoutData(new RowData(GUIDef.NTTAB_OPE_COMBO_X, GUIDef.NTTAB_OPE_COMBO_Y));
		opeCombo2.add(GUIDef.NTTAB_OPE_RRCMD[language]);
		opeCombo2.add(GUIDef.NTTAB_OPE_RLCMD[language]);
		opeCombo2.add(GUIDef.NTTAB_OPE_CFRCMD[language]);
		opeCombo2.add(GUIDef.NTTAB_OPE_CFLCMD[language]);
		opeCombo2.add(GUIDef.NTTAB_OPE_CBRCMD[language]);
		opeCombo2.add(GUIDef.NTTAB_OPE_CBLCMD[language]);
		opeCombo2.setText(GUIDef.NTTAB_OPE_RRCMD[language]);
		opeCombo2.select(0);
		opeSp21 = new Spinner(opeComp2, SWT.BORDER);
		opeSp21.setMinimum(RobominDef.NTPOWER_MIN);
		opeSp21.setMaximum(RobominDef.NTPOWER_MAX);
		opeSp21.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp21.setToolTipText(GUIDef.NTTAB_OPESP21_TOOLTIP[language]);
		opeSp22 = new Spinner(opeComp2, SWT.BORDER);
		opeSp22.setMinimum(RobominDef.NTANGLE_MIN);
		opeSp22.setMaximum(RobominDef.NTANGLE_MAX);
		opeSp22.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp22.setToolTipText(GUIDef.NTTAB_OPESP22_TOOLTIP[language]);
		opeComp21 = new Composite(opeComp2, SWT.NONE);
		opeComp21.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPACE_X-1, GUIDef.NTTAB_OPE_SPACE_Y));
		opeBtn2 = new Button(opeComp2, SWT.NONE);
		opeBtn2.setText(GUIDef.NTTAB_OPE_SENDBTN[language]);
		opeBtn2.setLayoutData(new RowData(GUIDef.NTTAB_OPE_BTN_X, GUIDef.NTTAB_OPE_BTN_Y));
		opeBtn2.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Rotation Command");
				
				if (rInfo != null) {
					int res = 0;
					
					switch (opeCombo2.getSelectionIndex()) {
					case 0:
						res = ntCmd.sendCmdRRotate(rInfo, opeSp21.getSelection(), opeSp22.getSelection(), -1, -1, -1);
						break;
					case 1:
						res = ntCmd.sendCmdLRotate(rInfo, opeSp21.getSelection(), opeSp22.getSelection(), -1, -1, -1);
						break;
					case 2:
						res = ntCmd.sendCmdRFCurve(rInfo, opeSp21.getSelection(), opeSp22.getSelection(), -1, -1, -1);
						break;
					case 3:
						res = ntCmd.sendCmdLFCurve(rInfo, opeSp21.getSelection(), opeSp22.getSelection(), -1, -1, -1);
						break;
					case 4:
						res = ntCmd.sendCmdRBCurve(rInfo, opeSp21.getSelection(), opeSp22.getSelection(), -1, -1, -1);
						break;
					case 5:
						res = ntCmd.sendCmdLBCurve(rInfo, opeSp21.getSelection(), opeSp22.getSelection(), -1, -1, -1);
						break;
					default:
						logger.log(Level.SEVERE, "There is no selection");
						break;
					}
					
					if (res < 0) {
						logger.log(Level.WARNING, "Send Rotation command error ErrorCode(" + res + ")");
					}
				}
			}
		});
		
		// ~R}hpl
		Group opeGroup3 = new Group(opeGroup, SWT.NONE);
		opeGroup3.setLayout(new FillLayout());
		opeComp3 = new Composite(opeGroup3, SWT.NONE);
		opeComp3.setLayout(new RowLayout(SWT.HORIZONTAL));
		opeCombo3 = new Combo(opeComp3, SWT.NONE);
		opeCombo3.add(GUIDef.NTTAB_OPE_STCMD[language]);
		opeCombo3.setText(GUIDef.NTTAB_OPE_STCMD[language]);
		opeCombo3.setLayoutData(new RowData(GUIDef.NTTAB_OPE_COMBO_X, GUIDef.NTTAB_OPE_COMBO_Y));	
		opeCombo3.select(0);
		opeComp31 = new Composite(opeComp3, SWT.NONE);
		opeComp31.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPACE_X*3, GUIDef.NTTAB_OPE_SPACE_Y));
		opeBtn3 = new Button(opeComp3, SWT.NONE);
		opeBtn3.setText(GUIDef.NTTAB_OPE_SENDBTN[language]);
		opeBtn3.setLayoutData(new RowData(GUIDef.NTTAB_OPE_BTN_X, GUIDef.NTTAB_OPE_BTN_Y));
		opeBtn3.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Stop Command");
				
				if (rInfo != null) {
					int res = 0;
					
					switch (opeCombo3.getSelectionIndex()) {
					case 0:
						res = ntCmd.sendCmdStop(rInfo);
						break;
					default:
						logger.log(Level.SEVERE, "There is no selection");
						break;
					}
					
					if (res < 0) {
						logger.log(Level.WARNING, "Send Stop command error ErrorCode(" + res + ")");
					}
				}
			}
		});
		
		// ߃R}hpl
		Group opeGroup4 = new Group(opeGroup, SWT.NONE);
		opeGroup4.setLayout(new FillLayout());
		opeComp4 = new Composite(opeGroup4, SWT.NONE);
		opeComp4.setLayout(new RowLayout(SWT.HORIZONTAL));
		opeCombo4 = new Combo(opeComp4, SWT.NONE);
		opeCombo4.setLayoutData(new RowData(GUIDef.NTTAB_OPE_COMBO_X, GUIDef.NTTAB_OPE_COMBO_Y));
		opeCombo4.add(GUIDef.NTTAB_OPE_EXCMD[language]);
		opeCombo4.add(GUIDef.NTTAB_OPE_FOCMD[language]);
		opeCombo4.add(GUIDef.NTTAB_OPE_WACMD[language]);
		opeCombo4.setText(GUIDef.NTTAB_OPE_EXCMD[language]);
		opeCombo4.select(0);
		opeSp41 = new Spinner(opeComp4, SWT.BORDER);
		opeSp41.setMinimum(RobominDef.NTPOWER_MIN);
		opeSp41.setMaximum(RobominDef.NTPOWER_MAX);
		opeSp41.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp41.setToolTipText(GUIDef.NTTAB_OPESP41_TOOLTIP[language]);
		opeSp42 = new Spinner(opeComp4, SWT.BORDER);
		opeSp42.setMinimum(RobominDef.NTSENSOR_MIN);
		opeSp42.setMaximum(RobominDef.NTSENSOR_MAX);
		opeSp42.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp42.setToolTipText(GUIDef.NTTAB_OPESP42_TOOLTIP[language]);
		opeSp43 = new Spinner(opeComp4, SWT.BORDER);
		opeSp43.setMinimum(RobominDef.NTTIME_MIN);
		opeSp43.setMaximum(RobominDef.NTTIME_MAX);
		opeSp43.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp43.setToolTipText(GUIDef.NTTAB_OPESP43_TOOLTIP[language]);
		opeBtn4 = new Button(opeComp4, SWT.NONE);
		opeBtn4.setText(GUIDef.NTTAB_OPE_SENDBTN[language]);
		opeBtn4.setLayoutData(new RowData(GUIDef.NTTAB_OPE_BTN_X, GUIDef.NTTAB_OPE_BTN_Y));
		opeBtn4.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Operation Command");
				
				if (rInfo != null) {
					int res = 0;
					
					switch (opeCombo4.getSelectionIndex()) {
					case 0:
						res = ntCmd.sendCmdExplore(rInfo, opeSp41.getSelection(), opeSp42.getSelection(), opeSp43.getSelection());
						break;
					case 1:
						res = ntCmd.sendCmdFollow(rInfo, opeSp41.getSelection(), opeSp42.getSelection(), opeSp43.getSelection());
						break;
					case 2:
						res = ntCmd.sendCmdWatch(rInfo, opeSp41.getSelection(), opeSp42.getSelection(), opeSp43.getSelection());
						break;
					default:
						logger.log(Level.SEVERE, "There is no selection");
						break;
					}
					
					if (res < 0) {
						logger.log(Level.WARNING, "Send Operation command error ErrorCode(" + res + ")");
					}
				}
			}
		});
		
		// sR}hpl
		Group opeGroup5 = new Group(opeGroup, SWT.NONE);
		opeGroup5.setLayout(new FillLayout());
		opeComp5 = new Composite(opeGroup5, SWT.NONE);
		opeComp5.setLayout(new RowLayout(SWT.HORIZONTAL));
		opeCombo5 = new Combo(opeComp5, SWT.NONE);
		opeCombo5.setLayoutData(new RowData(GUIDef.NTTAB_OPE_COMBO_X, GUIDef.NTTAB_OPE_COMBO_Y));
		opeCombo5.add(GUIDef.NTTAB_OPE_BRCMD[language]);
		opeCombo5.add(GUIDef.NTTAB_OPE_DACMD[language]);
		opeCombo5.setText(GUIDef.NTTAB_OPE_BRCMD[language]);
		opeCombo5.select(0);
		opeSp51 = new Spinner(opeComp5, SWT.BORDER);
		opeSp51.setMinimum(RobominDef.NTPOWER_MIN);
		opeSp51.setMaximum(RobominDef.NTPOWER_MAX);
		opeSp51.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp51.setToolTipText(GUIDef.NTTAB_OPESP51_TOOLTIP[language]);
		opeSp52 = new Spinner(opeComp5, SWT.BORDER);
		opeSp52.setMinimum(RobominDef.NTTIME_MIN);
		opeSp52.setMaximum(RobominDef.NTTIME_MAX);
		opeSp52.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp52.setToolTipText(GUIDef.NTTAB_OPESP52_TOOLTIP[language]);
		opeComp51 = new Composite(opeComp5, SWT.NONE);
		opeComp51.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPACE_X-1, GUIDef.NTTAB_OPE_SPACE_Y));
		opeBtn5 = new Button(opeComp5, SWT.NONE);
		opeBtn5.setText(GUIDef.NTTAB_OPE_SENDBTN[language]);
		opeBtn5.setLayoutData(new RowData(GUIDef.NTTAB_OPE_BTN_X, GUIDef.NTTAB_OPE_BTN_Y));
		opeBtn5.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Run Command");
				
				if (rInfo != null) {
					int res = 0;
					
					switch (opeCombo5.getSelectionIndex()) {
					case 0:
						res = ntCmd.sendCmdBright(rInfo, opeSp51.getSelection(), opeSp52.getSelection());
						break;
					case 1:
						res = ntCmd.sendCmdDark(rInfo, opeSp51.getSelection(), opeSp52.getSelection());
						break;
					default:
						logger.log(Level.SEVERE, "There is no selection");
						break;
					}
					
					if (res < 0) {
						logger.log(Level.WARNING, "Send Run command error ErrorCode(" + res + ")");
					}
				}
			}
		});
		
		// [^[R}hpl
		Group opeGroup6 = new Group(opeGroup, SWT.NONE);
		opeGroup6.setLayout(new FillLayout());
		opeComp6 = new Composite(opeGroup6, SWT.NONE);
		opeComp6.setLayout(new RowLayout(SWT.HORIZONTAL));
		opeCombo6 = new Combo(opeComp6, SWT.NONE);
		opeCombo6.setLayoutData(new RowData(GUIDef.NTTAB_OPE_COMBO_X, GUIDef.NTTAB_OPE_COMBO_Y));
		opeCombo6.add(GUIDef.NTTAB_OPE_MMCMD[language]);
		opeCombo6.setText(GUIDef.NTTAB_OPE_MMCMD[language]);
		opeCombo6.select(0);
		opeSp61 = new Spinner(opeComp6, SWT.BORDER);
		opeSp61.setMinimum(RobominDef.NTMOTORP_MIN);
		opeSp61.setMaximum(RobominDef.NTMOTORP_MAX);
		opeSp61.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp61.setToolTipText(GUIDef.NTTAB_OPESP61_TOOLTIP[language]);
		opeSp62 = new Spinner(opeComp6, SWT.BORDER);
		opeSp62.setMinimum(RobominDef.NTMOTORP_MIN);
		opeSp62.setMaximum(RobominDef.NTMOTORP_MAX);
		opeSp62.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPIN_X, GUIDef.NTTAB_OPE_SPIN_Y));
		opeSp62.setToolTipText(GUIDef.NTTAB_OPESP62_TOOLTIP[language]);
		opeComp61 = new Composite(opeComp6, SWT.NONE);
		opeComp61.setLayoutData(new RowData(GUIDef.NTTAB_OPE_SPACE_X-1, GUIDef.NTTAB_OPE_SPACE_Y));
		opeBtn6 = new Button(opeComp6, SWT.NONE);
		opeBtn6.setText(GUIDef.NTTAB_OPE_SENDBTN[language]);
		opeBtn6.setLayoutData(new RowData(GUIDef.NTTAB_OPE_BTN_X, GUIDef.NTTAB_OPE_BTN_Y));
		opeBtn6.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Motor Command");
				
				if (rInfo != null) {
					int res = 0;
					
					switch (opeCombo6.getSelectionIndex()) {
					case 0:
						res = ntCmd.sendCmdMM(rInfo, opeSp61.getSelection(), opeSp62.getSelection());
						break;
					default:
						logger.log(Level.SEVERE, "There is no selection");
						break;
					}
					
					if (res < 0) {
						logger.log(Level.WARNING, "Send Motor command error ErrorCode(" + res + ")");
					}
				}
			}
		});
		
		// Route Indicator
		routeGroup = new Group(subSash, SWT.SHADOW_IN);
		routeGroup.setText(GUIDef.NTTAB_ROUTEGROUP[language]);
		routeGroup.setLayout(new GridLayout(1, true));
		routeGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		subSash.setWeights(new int[] {5, 5});
		
		routeComp = new Composite(routeGroup, SWT.NONE);
		routeComp.setLayout(new FillLayout(SWT.HORIZONTAL));
		routeComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		routeStartBtn = new Button(routeComp, SWT.NONE);
		routeStartBtn.setText(GUIDef.NTTAB_ROUTE_STARTBTN[language]);
		routeStopBtn = new Button(routeComp, SWT.NONE);
		routeStopBtn.setText(GUIDef.NTTAB_ROUTE_STOPBTN[language]);
		routeStopBtn.setEnabled(false);
		routeResetBtn = new Button(routeComp, SWT.NONE);
		routeResetBtn.setText(GUIDef.NTTAB_ROUTE_RESETBTN[language]);
		routeResetBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				prePoint.x = 0;
				prePoint.y = 0;
				routeCanvas.redraw();
			}
		});
		
		prePoint = new Point(0,0);
		routeCanvas = new Canvas(routeGroup, SWT.BACKGROUND);
		routeCanvas.setLayoutData(new GridData(GridData.FILL_BOTH));
		routeCanvas.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));
		routeCanvas.addListener(SWT.MouseDown, new Listener() {
			public void handleEvent(Event e) {
				if (e.button == 1) {
					GC gc = new GC(routeCanvas);
					gc.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_BLUE));
					gc.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_BLUE));
					gc.fillOval(e.x, e.y, 10, 10);
					if (prePoint.x != 0 && prePoint.y != 0) {
						gc.drawLine(prePoint.x + 5, prePoint.y + 5, e.x + 5,
								e.y + 5);

					}
					else {
						gc.drawImage(tabIcon, e.x-tabIcon.getBounds().width/2, e.y-tabIcon.getBounds().height/2);
					}
					gc.dispose();
					prePoint.x = e.x;
					prePoint.y = e.y;
				}
			}
		});
		
		// Phidget Sensor
		phidgetGroup = new Group(subSash2, SWT.SHADOW_IN);
		phidgetGroup.setText(GUIDef.NTTAB_PHIDGETGROUP[language]);
		phidgetGroup.setLayout(new GridLayout(1, true));
		phidgetGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		phidgetComp1 = new Composite(phidgetGroup, SWT.SHADOW_IN);
		phidgetComp1.setLayout(new GridLayout(3, true));
		phidgetComp1.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		// VKڑ{^
		phidgetNewConBtn = new Button(phidgetComp1, SWT.PUSH);
		phidgetNewConBtn.setImage(phidgetNewConIcon);
		phidgetNewConBtn.setToolTipText(GUIDef.NTTAB_PHDGET_TOOL1[language]);
		phidgetNewConBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				// EBU[h̍쐬
				PhidgetWizard pWiz = new PhidgetWizard(info);
                WizardDialog dialog = new WizardDialog(info.getRobominGUI().getShell(), pWiz);
                WizardDialog.setDefaultImage(phidgetReConIcon);

                int res = dialog.open();

                if (res == WizardDialog.OK) {
                	int res2 = 0;
                	
                	// [JPCPhidgetZT[ڑꍇ
                	if (pWiz.phidgetDest == PhidgetDef.DEST_LOCAL) {
                		pInfo = new PhidgetInfo(PhidgetDef.DEST_LOCAL);
	    				res2 = pIKP.waitForAttachment(PhidgetTimeout);
                	}
                	// [gPCPhidgetZT[ڑꍇ
                	else {
                		pInfo = new PhidgetInfo(PhidgetDef.DEST_REMOTE, pWiz.serialNumber, pWiz.ipAddr, pWiz.portNumber, pWiz.passwd);
                		res2 = pIKP.waitForAttachment(pWiz.serialNumber, pWiz.ipAddr, pWiz.portNumber, pWiz.passwd, PhidgetTimeout);
                	}
                	
                	pInfo.setJoyXIO(pWiz.joyXIO);
                	pInfo.setJoyYIO(pWiz.joyYIO);
	    				
                	if (res2 == RobominDef.SUCCESS) {
	    				phidgetDisBtn.setEnabled(true);
	    				phidgetNewConBtn.setEnabled(false);
	    				phidgetReConBtn.setEnabled(false);
                	}
                	else {
                		pIKP.close();
                	}
                }
			}
		});
		
		// Đڑ{^
		phidgetReConBtn = new Button(phidgetComp1, SWT.PUSH);
		phidgetReConBtn.setImage(phidgetReConIcon);
		phidgetReConBtn.setToolTipText(GUIDef.NTTAB_PHDGET_TOOL2[language]);
		phidgetReConBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (pInfo != null) {
					int res;
					if (pInfo.getSensorDest() == PhidgetDef.DEST_LOCAL) {
						res = pIKP.waitForAttachment(PhidgetTimeout);
					}
					else {
						res = pIKP.waitForAttachment(pInfo, PhidgetTimeout);
					}
					
                	if (res == RobominDef.SUCCESS) {
	    				phidgetDisBtn.setEnabled(true);
	    				phidgetNewConBtn.setEnabled(false);
	    				phidgetReConBtn.setEnabled(false);
                	}
                	else {
                		pIKP.close();
                	}
				}
			}
		});
		
		// ؒf{^
		phidgetDisBtn = new Button(phidgetComp1, SWT.PUSH);
		phidgetDisBtn.setEnabled(false);
		phidgetDisBtn.setImage(phidgetDisBtnIcon);
		phidgetDisBtn.setToolTipText(GUIDef.NTTAB_PHDGET_TOOL3[language]);
		phidgetDisBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (pIKP.close() == RobominDef.SUCCESS) {
					phidgetNewConBtn.setEnabled(true);
					phidgetDisBtn.setEnabled(false);
					phidgetReConBtn.setEnabled(true);
				}
			}
		});
		
		phidgetComp2 = new Composite(phidgetGroup, SWT.SHADOW_IN);
		phidgetComp2.setLayout(new GridLayout(2, true));
		phidgetComp2.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		infoFont = new Font(Display.getDefault(), "Tahoma", 10, SWT.BOLD);
		
		phidgetJoyGroup = new Group(phidgetComp2, SWT.SHADOW_IN);
		phidgetJoyGroup.setText(GUIDef.NTTAB_PHIDGET_JOYGROUP[language]);
		phidgetJoyGroup.setLayout(new GridLayout(1, true));
		phidgetJoyGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		phidgetJoyLabel = new Label(phidgetJoyGroup, SWT.CENTER);
		phidgetJoyLabel.setLayoutData(new GridData(GridData.FILL_BOTH));
		phidgetJoyLabel.setFont(infoFont);
		
		phidgetSliderGroup = new Group(phidgetComp2, SWT.SHADOW_IN);
		phidgetSliderGroup.setText(GUIDef.NTTAB_PHIDGET_SLIDER[language]);
		phidgetSliderGroup.setLayout(new GridLayout(1, true));
		phidgetSliderGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		phidgetSliderLabel = new Label(phidgetSliderGroup, SWT.CENTER);
		phidgetSliderLabel.setLayoutData(new GridData(GridData.FILL_BOTH));
		phidgetSliderLabel.setFont(infoFont);
		
		// Robomin Prompt
		promptGroup = new Group(subSash2, SWT.SHADOW_IN);
		promptGroup.setText(GUIDef.NTTAB_PROMPTGROUP[language]);
		promptGroup.setLayout(new GridLayout(1, true));
		promptGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		mainSash.setWeights(new int[] {7, 3});
		tabItem.setControl(mainSash);
		
		promptText = new Text(promptGroup, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL);
		promptText.setLayoutData(new GridData(GridData.FILL_BOTH));
		promptText.addKeyListener(new KeyListener() {
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.CR) {
					promptText.append("prompt>");
				}
			}
			
			public void keyPressed(KeyEvent e) {
				
			}
		});
		
		subSash2.setWeights(new int[] {5, 5});
		
		ntCmd = info.getNetTansorCmd();
	}
	
	/**
	 * RobotInfoZbg
	 * @param rInfo ZbgRobotInfo
	 */
	public void setRobotInfo(RobotInfo rInfo) {
		this.rInfo = rInfo;
	}
	
	/**
	 * PhidgetZT[ڑꂽƂɌĂ΂
	 */
	public void attached(AttachEvent event) {
		logger.log(Level.INFO, "Attach " + event);
	}
	
	/**
	 * PhidgetZT[ؒfꂽƂɌĂ΂
	 */
	public void detached(DetachEvent event) {
		logger.log(Level.INFO, "Detach " + event);
	}
	
	/**
	 * PhidgetZT[G[̎ɌĂ΂
	 */
	public void error(ErrorEvent event) {
		logger.log(Level.SEVERE, "Error " + event);
	}
	
	/**
	 * PhidgetZT[̓͂ςƂɌĂ΂
	 */
	public void inputChanged(InputChangeEvent event) {
		logger.log(Level.INFO, "Input " + event);
	}
	
	/**
	 * PhidgetZT[̏o͂ςƂɌĂ΂
	 */
	public void outputChanged(OutputChangeEvent event) {
		logger.log(Level.INFO, "Output " + event);
	}
	
	/**
	 * PhidgetZT[̒lςƂɌĂ΂
	 */
	public void sensorChanged(SensorChangeEvent event) {
		
		// ZT[Ԃ񂩂IO|[gƒl擾
		int sIO = pIKP.getSensorIndex(event.toString());
		int sValue = pIKP.getSensorValue(event.toString());
		
		logger.log(Level.CONFIG, "SensorChange " + event);
		
		// lۑ
		pInfo.saveSValue(sIO, sValue);
		
		// ZT[Ƃ͈̏ȉɒǉďs
		
		// WCXeBbNZT[
		if (sIO == pInfo.getJoyXIO() || sIO == pInfo.getJoyYIO()) {
			
			phidgetJoyX = pInfo.getSValue(pInfo.getJoyXIO());
			phidgetJoyY = pInfo.getSValue(pInfo.getJoyYIO());
			
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					phidgetJoyLabel.setText("(x, y) = " + phidgetJoyX + ", " + phidgetJoyY);
				}
			});
		}
		// XC_[
		else if (sIO == pInfo.getSliderIO()) {
			
			phidgetSliderV = pInfo.getSValue(pInfo.getSliderIO());
			
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					phidgetSliderLabel.setText("(value) = " + phidgetSliderV);
				}
			});
		}
	}
}
