/**
 * Program RobominGUI.java
 * 
 * Date         Version    Author         Comment
 * 2008/03/17   1.0.0      T.Kawashima    First.
 * 
 */

package robomin.ui;

import java.util.logging.Level;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import robomin.robot.RobotInfo;
import robomin.ui.listener.RobominCloseListener;
import robomin.util.RobominLogger;
import robomin.util.RobominDef;
import robomin.util.RobominInfo;

/**
 * TansorClientGUI쐬NX
 * @author T.Kawashima
 * @version 1.0.0
 */
public class RobominGUI {
	private RobominInfo info;
	private RobominLogger logger;
	private int language;
	
	private Display display;
	private Shell shell;
	
	// ACRpC[W
	private Image appliIcon;
	private Image searchIcon;
	private Image registerIcon;
	private Image deleteIcon;
	private Image connectIcon;
	private Image disconnectIcon;
	private Image infoIcon;
	private ImageRegistry imgReg;
	
	// j[
	private Menu barMenu;
	
	private Menu fileMenu;
	private MenuItem fileMenuItem;
	private MenuItem file_quitMenuItem;
	
	private Menu opeMenu;
	private MenuItem opeMenuItem;
	private MenuItem ope_sercMenuItem;
	private MenuItem ope_registMenuItem;
	private MenuItem ope_delMenuItem;
	private MenuItem ope_connMenuItem;
	private MenuItem ope_disconnMenuItem;
	
	private Menu setMenu;
	private MenuItem setMenuItem;
	private MenuItem set_fpsMenuItem;
	private MenuItem set_langMenuItem;
	private Menu set_langSubMenu;
	private MenuItem set_langJpMenuItem;
	private MenuItem set_langEnMenuItem;
	private MenuItem set_logMenuItem;
	private Menu set_logSubMenu;
	private MenuItem[] set_logSubMenuItems;

	private Menu helpMenu;
	private MenuItem helpMenuItem;
	private MenuItem aboutMenuItem;
	
	// |bvAbvj[
	private Menu popup_leafMenu;
	private MenuItem popup_connMenuItem;
	private MenuItem popup_disconnMenuItem;
	private MenuItem popup_delMenuItem;
	
	private Menu popup_rootMenu;
	private MenuItem popup_registMenuItem;
	
	// c[o[
	private ToolBar toolBar;
	private ToolItem searchItem;
	private ToolItem registerItem;
	private ToolItem deleteItem;
	private ToolItem connectItem;
	private ToolItem disconnectItem;
	private ToolItem infoItem;
	
	private SashForm mainSashForm;
	private SashForm leftAreaSashForm;
	
	// pl
	private Group infoGroup;
	private Label infoLabel;
	private Font infoFont;
	
	// J
	private Group cameraGroup;
	public Canvas cameraCanvas;
	private Image cameraImg;
	private Image backgroundImg;
	
	// c[
	private Tree tree;
	private TreeItem root;
	
	// ^u
	private CTabFolder tabFolder;
	public NetTansorTab nettansorTab;
	public MINDSTORMTab mindstormTab;
	public ConsoleTab consoleTab;
	
	// J\XbhptO
	private boolean threadFlag = true;
	
	// ڑؒf{^̃gOtO
	private boolean condisconFlag = true;
	
	// JFPS
	private int cameraFps = 10;
	private int cameraFpsTemp = 10;
	private int redrawTime = 1000;
	
	// _CAOƂ̏󂯓nɎgp
	private RobotInfo rInfoTemp;
	
	// ݐڑĂ郍{bgRobotInfoۑ
	private RobotInfo rInfoCurrent = null;
	
	/**
	 * RXgN^
	 * 
	 * @param title
	 *            EBhẼ^Cgw肷
	 * @param info
	 *            ʏۑNX
	 */
	public RobominGUI(String title, final RobominInfo info) {
		this.info = info;
		this.logger = info.getRobominLogger();
		language = info.getLanguage();
		display = new Display();
		
		//createSplashWindow();
		
		shell = new Shell(display);
		shell.setText(title);
		shell.setLayout(new GridLayout(1, true));
		shell.setSize(960, 650);
		shell.addListener(SWT.Close, new RobominCloseListener(info));
		initIcons();
		shell.setImage(appliIcon);
		
		createMenuBar();
		createToolBar();
		createPopupRootMenu();
		createPopupLeafMenu();
		createMainSash();
	}
	
	/**
	 * XvbVEBhE̍쐬E\
	 */
    private void createSplashWindow() {
        Shell splash = new Shell(SWT.ON_TOP);
        splash.setLayout(new GridLayout(1, false));
        Image splashImage = new Image(display, GUIDef.SPLASH_ICONPATH);
        Label splashLabel = new Label(splash, SWT.NONE);
        splashLabel.setImage(splashImage);

        Label loadLabel = new Label(splash, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        loadLabel.setLayoutData(gd);

        splash.pack();

        //XvbVEBhE𒆉ɔzu
        Rectangle shellRect = splash.getBounds();
        Rectangle dispRect = display.getBounds();
        int x = (dispRect.width - shellRect.width) / 2;
        int y = (dispRect.height - shellRect.height) / 2;

        splash.setLocation(x, y);
        splash.open();

        // ʂɎԂȂĂǂ͋̕Cł̂ŁEEE
        for (int i = 0; i < 10; i++) {
                loadLabel.setText(GUIDef.SPLASH_LAUCH_MSG[language] + i * 10 + "%");
                try {
                        Thread.sleep(100);
                } catch (InterruptedException e) {
                        e.printStackTrace();
                }
        }
        
        loadLabel.setText(GUIDef.SPLASH_COMP_MSG[language]);
        splashImage.dispose();
        splash.dispose();
    }
	
	/**
	 * ACR摜(PNG)ǂݍImage
	 */
	public void initIcons() {
		imgReg = new ImageRegistry();
		
		try {
			appliIcon = new Image(display, GUIDef.APPLICATION_ICONPATH);
			imgReg.put(GUIDef.APPLICATION_ICONPATH, appliIcon);
			
			searchIcon = new Image(display, GUIDef.SEARCH_ICONPATH );
			imgReg.put(GUIDef.SEARCH_ICONPATH, searchIcon);
			
			registerIcon = new Image(display, GUIDef.REGISTER_ICONPATH );
			imgReg.put(GUIDef.REGISTER_ICONPATH, registerIcon);
			
			deleteIcon = new Image(display, GUIDef.DELETE_ICONPATH );
			imgReg.put(GUIDef.DELETE_ICONPATH, deleteIcon);
			
			connectIcon = new Image(display, GUIDef.CONNECT_ICONPATH);
			imgReg.put(GUIDef.CONNECT_ICONPATH, connectIcon);
			
			disconnectIcon = new Image(display, GUIDef.DISCONNECT_ICONPATH);
			imgReg.put(GUIDef.DISCONNECT_ICONPATH, disconnectIcon);
			
			infoIcon = new Image(display, GUIDef.INFO_ICONPATH);
			imgReg.put(GUIDef.INFO_ICONPATH, infoIcon);
		}
		catch (SWTException e) {
			logger.log(Level.SEVERE, "Error", e);
		}
	}
	
	/**
	 * MenuBar쐬
	 */
	private void createMenuBar() {
		barMenu = new Menu(shell, SWT.BAR);
		shell.setMenuBar(barMenu);
		
		// j[ t@C
		fileMenuItem = new MenuItem(barMenu, SWT.CASCADE);
		fileMenuItem.setText(GUIDef.MENU_FILE[language]);
		fileMenu = new Menu(fileMenuItem);
		fileMenuItem.setMenu(fileMenu);

		// j[ I
		file_quitMenuItem = new MenuItem(fileMenu, SWT.PUSH);
		file_quitMenuItem.setText(GUIDef.MENU_FILE_QUIT[language]);
		file_quitMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Menu Quit selected");
				System.exit(0);
			}
		});

		// j[ 
		opeMenuItem = new MenuItem(barMenu, SWT.CASCADE);
		opeMenuItem.setText(GUIDef.MENU_OPE[language]);
		opeMenu = new Menu(opeMenuItem);
		opeMenuItem.setMenu(opeMenu);
		
		// j[@lbg^T[
		ope_sercMenuItem = new MenuItem(opeMenu, SWT.PUSH);
		ope_sercMenuItem.setText(GUIDef.MENU_OPE_SERC[language]);
		ope_sercMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Menu Search selected");
				searchRobot();
			}
		});
		
		// j[@{bgo^
		ope_registMenuItem = new MenuItem(opeMenu, SWT.PUSH);
		ope_registMenuItem.setText(GUIDef.MENU_OPE_REG[language]);
		ope_registMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Menu Register selected");
				registerNewRobot();
			}
		});
		
		// j[@{bg폜
		ope_delMenuItem = new MenuItem(opeMenu, SWT.PUSH);
		ope_delMenuItem.setText(GUIDef.MENU_OPE_DEL[language]);
		ope_delMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Menu Delete selected");
				deleteRobot(rInfoCurrent);
			}
		});
		
		// j[  lbg^T[Ɛڑ
		ope_connMenuItem = new MenuItem(opeMenu, SWT.PUSH);
		ope_connMenuItem.setText(GUIDef.MENU_OPE_CONN[language]);
		ope_connMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Menu Connect selected");
				connectRobot(rInfoCurrent);
			}
		});
		
		// j[  lbg^T[Ɛؒf
		ope_disconnMenuItem = new MenuItem(opeMenu, SWT.PUSH);
		ope_disconnMenuItem.setText(GUIDef.MENU_OPE_DISCONN[language]);
		ope_disconnMenuItem.setEnabled(false);
		ope_disconnMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Menu Disconnect selected");
				disconnectRobot(rInfoCurrent);
			}
		});
		
		// j[ ݒ
		setMenuItem = new MenuItem(barMenu, SWT.CASCADE);
		setMenuItem.setText(GUIDef.MENU_SET[language]);
		setMenu = new Menu(setMenuItem);
		setMenuItem.setMenu(setMenu);
		
		// j[@FPS(J)̕ύX
		set_fpsMenuItem = new MenuItem(setMenu, SWT.PUSH);
		set_fpsMenuItem.setText(GUIDef.MENU_SET_FPS[language]);
		set_fpsMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				FpsChangeDialog fpsDialog = new FpsChangeDialog(shell);
				int res = fpsDialog.open();
				
				if (res == Dialog.OK) {
					logger.log(Level.CONFIG, "FPS change"  + cameraFps + " -> " + cameraFpsTemp);
					cameraFps = cameraFpsTemp;
					setCameraFps(cameraFps);
				}
			}
		});
		
		// j[@̕ύX
		set_langMenuItem = new MenuItem(setMenu, SWT.CASCADE);
		set_langMenuItem.setText(GUIDef.MENU_SET_LANG[language]);
		set_langSubMenu = new Menu(set_langMenuItem);
		set_langMenuItem.setMenu(set_langSubMenu);
		
		set_langJpMenuItem = new MenuItem(set_langSubMenu, SWT.RADIO);
		set_langJpMenuItem.setText(GUIDef.MENU_SET_LANG_JP[language]);
		if (language == RobominDef.LANG_JP) {
			set_langJpMenuItem.setSelection(true);
		}
		set_langJpMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (((MenuItem)e.widget).getSelection()) {
					changeSettingLanguage(RobominDef.LANG_JP);
				}
			}
		});
		
		set_langEnMenuItem = new MenuItem(set_langSubMenu, SWT.RADIO);
		set_langEnMenuItem.setText(GUIDef.MENU_SET_LANG_EN[language]);
		if (language == RobominDef.LANG_EN) {
			set_langEnMenuItem.setSelection(true);
		}
		set_langEnMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (((MenuItem)e.widget).getSelection()) {
					changeSettingLanguage(RobominDef.LANG_EN);
				}
			}
		});
		
		// j[@Ox̕ύX
		set_logMenuItem = new MenuItem(setMenu, SWT.CASCADE);
		set_logMenuItem.setText(GUIDef.MENU_SET_LOG[language]);
		set_logSubMenu = new Menu(set_langMenuItem);
		set_logMenuItem.setMenu(set_logSubMenu);
		
		set_logSubMenuItems = new MenuItem[RobominDef.LOGLEVEL_DEBUG+1];

		for (int i=0; i<=RobominDef.LOGLEVEL_DEBUG; i++) {
			set_logSubMenuItems[i] = new MenuItem(set_logSubMenu, SWT.RADIO);
			if (i == logger.getLogLevel()) {
				set_logSubMenuItems[i].setSelection(true);
			}
		}

		set_logSubMenuItems[RobominDef.LOGLEVEL_SEVERE].setText(GUIDef.MENU_SET_LOG_SE[language]);
		set_logSubMenuItems[RobominDef.LOGLEVEL_SEVERE].addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (((MenuItem)e.widget).getSelection()) {
					info.getRobominLogger().setLogLevel(RobominDef.LOGLEVEL_SEVERE);
					logger.log(Level.SEVERE, "Log level change to SEVERE");
				}
			}
		});
		
		set_logSubMenuItems[RobominDef.LOGLEVEL_WARNING].setText(GUIDef.MENU_SET_LOG_WA[language]);
		set_logSubMenuItems[RobominDef.LOGLEVEL_WARNING].addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (((MenuItem)e.widget).getSelection()) {
					info.getRobominLogger().setLogLevel(RobominDef.LOGLEVEL_WARNING);
					logger.log(Level.WARNING, "Log level change to WARNING");
				}
			}
		});
		
		set_logSubMenuItems[RobominDef.LOGLEVEL_INFO].setText(GUIDef.MENU_SET_LOG_IN[language]);
		set_logSubMenuItems[RobominDef.LOGLEVEL_INFO].addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (((MenuItem)e.widget).getSelection()) {
					info.getRobominLogger().setLogLevel(RobominDef.LOGLEVEL_INFO);
					logger.log(Level.INFO, "Log level change to INFO");
				}
			}
		});
		
		set_logSubMenuItems[RobominDef.LOGLEVEL_DEBUG].setText(GUIDef.MENU_SET_LOG_DE[language]);
		set_logSubMenuItems[RobominDef.LOGLEVEL_DEBUG].addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (((MenuItem)e.widget).getSelection()) {
					info.getRobominLogger().setLogLevel(RobominDef.LOGLEVEL_DEBUG);
					logger.log(Level.CONFIG, "Log level change to DEBUG");
				}
			}
		});
		
		// j[  wv
		helpMenuItem = new MenuItem(barMenu, SWT.CASCADE);
		helpMenuItem.setText(GUIDef.MENU_HELP[language]);
		helpMenu = new Menu(helpMenuItem);
		helpMenuItem.setMenu(helpMenu);
		
		// j[  Roboministratorɂ
		aboutMenuItem = new MenuItem(helpMenu, SWT.PUSH);
		aboutMenuItem.setText(GUIDef.MENU_HELP_ABOUT[language]);
		aboutMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Menu Help selected");
				showInfoDialog();
			}
		});
	}
	
	/**
	 * \ݒύX
	 * @param language ύX錾
	 */
	public void changeSettingLanguage(int language) {
		String msg = "";
		
		if (language == RobominDef.LANG_JP) msg = "Japanese";
		else if (language == RobominDef.LANG_EN) msg = "English";
		
		logger.log(Level.CONFIG, "Language change to "  + msg);
		info.setLanguage(language);
		
		showLangDialog();
	}
	
	/**
	 * [tp|bvAbvj[
	 */
	private void createPopupLeafMenu() {
		popup_leafMenu = new Menu(shell, SWT.POP_UP);
		
		// j[ ڑ
		popup_connMenuItem = new MenuItem(popup_leafMenu, SWT.PUSH);
		popup_connMenuItem.setText(GUIDef.MENU_OPE_CONN[language]);
		popup_connMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Popup Connect selected");
				connectRobot(rInfoCurrent);
			}
		});
		
		// j[  ؒf
		popup_disconnMenuItem = new MenuItem(popup_leafMenu, SWT.PUSH);
		popup_disconnMenuItem.setText(GUIDef.MENU_OPE_DISCONN[language]);
		popup_disconnMenuItem.setEnabled(false);
		popup_disconnMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Popup Disconnect selected");
				disconnectRobot(rInfoCurrent);
			}
		});
		
		// j[  폜
		popup_delMenuItem = new MenuItem(popup_leafMenu, SWT.PUSH);
		popup_delMenuItem.setText(GUIDef.MENU_OPE_DEL[language]);
		popup_delMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Popup Delete selected");
				deleteRobot(rInfoCurrent);
			}
		});
	}
	
	/**
	 * [gp|bvAbvj[
	 */
	private void createPopupRootMenu() {
		popup_rootMenu = new Menu(shell, SWT.POP_UP);
		
		// j[  o^
		popup_registMenuItem = new MenuItem(popup_rootMenu, SWT.PUSH);
		popup_registMenuItem.setText(GUIDef.MENU_OPE_REG[language]);
		popup_registMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Popup Register selected");
				registerNewRobot();
			}
		});
	}
	
	/**
	 * ڑAؒf{^̃gO
	 */
	private void toggleConDisConMenu() {
		logger.log(Level.CONFIG, "condisconFlag: " + condisconFlag);
		
		// ڑ{^ENABLE̎
		if (condisconFlag) {
			connectItem.setEnabled(false);
			disconnectItem.setEnabled(true);
			ope_connMenuItem.setEnabled(false);
			ope_disconnMenuItem.setEnabled(true);
			popup_connMenuItem.setEnabled(false);
			popup_disconnMenuItem.setEnabled(true);
			condisconFlag = false;
		}
		// ڑ{^DISABLE̎
		else {
			connectItem.setEnabled(true);
			disconnectItem.setEnabled(false);
			ope_connMenuItem.setEnabled(true);
			ope_disconnMenuItem.setEnabled(false);
			popup_connMenuItem.setEnabled(true);
			popup_disconnMenuItem.setEnabled(false);
			condisconFlag = true;
		}
	}
	
	/**
	 * InfoLabelɃ{bgLڂ
	 * @param rInfo LڂRobotInfo
	 */
	public void writeInfoLabel(RobotInfo rInfo) {
		if (rInfo != null) {
			infoLabel.setText(rInfo.getName() + " (" + rInfo.getAddr() + ")");
		}
		else {
			infoLabel.setText(GUIDef.GROUP_INFO_DEFAULT[language]);
		}
	}
	
	/**
	 * VKɃ{bgo^
	 * @return o^0Ao^s-1AڑeXg-2
	 */
	private int registerNewRobot() {
		int res = showRegisterDialog();
		
		if (res == Dialog.OK) {
			// infoxɐڑAhX\
			writeInfoLabel(rInfoTemp);
			
			// ڑeXg
			int res2 = rInfoTemp.connectRobot();

			if (res2 < 0) {
				logger.log(Level.WARNING, "Connect Fail: ErrorCode(" + res2 + ") " + rInfoTemp.getAddr());
				// G[bZ[W_CAO\
				showConnectFailDialog(res2);
				writeInfoLabel(rInfoCurrent);
				return RobominDef.FAIL_CONNECT;
			}
			
			rInfoTemp.disconnectRobot();
		
			// ڑeXgɐc[֒ǉ
			int res3 = addRobotInfo(rInfoTemp);

			if (res3 < 0) {
				logger.log(Level.WARNING, "Add Fail: ErrorCode(" + res3 + ") " + rInfoTemp.getName() + " (" + rInfoTemp.getAddr() + ")");
				// G[bZ[W_CAO\
				showAddFailDialog(res3);
				writeInfoLabel(rInfoCurrent);
				return RobominDef.FAIL_SIZE;
			}
			
			logger.log(Level.INFO, "Connection Test Success " + rInfoTemp.getName() + " (" + rInfoTemp.getAddr() + ")");
			
			return RobominDef.SUCCESS;
		}
		
		logger.log(Level.CONFIG, "RegisterDialog canceled");
		
		return RobominDef.CANCEL;
	}
	
	/**
	 * {bg폜
	 * @param rInfo 폜RobotInfo
	 * @return 0As<0
	 */
	private int deleteRobot(RobotInfo rInfo) {
		
		if (rInfo == null) return 0;
		
		// ڑ̏ꍇ͐ؒf
		if (!rInfo.isClosed()) {
			int res = disconnectRobot(rInfo);
			
			if (res < 0) {
				logger.log(Level.WARNING, "Delete failed: ErrorCode(" + res + ") " + rInfo.getName() + " (" + rInfo.getAddr() + ")");
				return res;
			}
		}

		// Xg폜
		int index = info.removeRobotInfo(rInfo);
		
		if(index == RobominDef.FAIL) {
			return RobominDef.FAIL;
		}
		
		logger.log(Level.INFO, "Delete " + rInfo.getName() + " (" + rInfo.getAddr() + ")");
		logger.log(Level.CONFIG, "Delete (" + index + ")");
		
		// c[폜
		root.getItem(index).dispose();
		
		return 0;
	}
	
	/**
	 * {bg()
	 * @return 0As-1
	 */
	private int searchRobot() {
		int res = 0;
		
		return res;
	}
	
	/**
	 * {bgƃRlNVm
	 * @param rInfo ڑ郍{bgRobotInfo
	 * @return 0As<0
	 */
	public int connectRobot(RobotInfo rInfo) {
		// ڑEؒf{^̃gO
		toggleConDisConMenu();
		
		// {bgo^̏ꍇ
		if (rInfo == null) {
			int res = registerNewRobot();
			
			// o^sꍇ
			if (res < 0) {
				if (res != RobominDef.CANCEL) {
					logger.log(Level.WARNING, "registerNewRobot failed: ErrorCode(" + res + ") " + rInfoTemp.getName() + " (" + rInfoTemp.getAddr() + ")");
				}
				
				toggleConDisConMenu();
				return res;
			}
			
			rInfo = rInfoTemp;
		}

		// lbg^T[ƃRlNVm
		int res = rInfo.connectRobot();
		
		if (res < 0) {
			logger.log(Level.WARNING, "Connect Fail: ErrorCode(" + res + ") " + rInfo.getName() + " (" + rInfo.getAddr() + ")");
			// G[bZ[W_CAO\
			showConnectFailDialog(res);
			
			// ڑEؒf{^̃gO
			toggleConDisConMenu();
			return res;
		}
		
		rInfoCurrent = rInfo;
		
		logger.log(Level.INFO, "Connect Successed: " + rInfo.getName() + " (" + rInfo.getAddr() + ")");
		
		// J摜擾XbhX^[g
		info.getRobominGUI().cameraThreadStart();
		
		nettansorTab.setRobotInfo(rInfoCurrent);
		
		return res;
	}
	
	/**
	 * lbg^T[ƃRlNVؒf
	 * @return 0As-1
	 */
	public int disconnectRobot(RobotInfo rInfo) {
		// J摜擾XbhXgbv
		info.getRobominGUI().cameraThreadStop();
		
		int res = rInfo.disconnectRobot();

		if (res < 0) {
			logger.log(Level.WARNING, "Disconnect Fail: ErrorCode(" + res + ") " + rInfo.getName() + " (" + rInfo.getAddr() + ")");
			
			// G[_CAO\邾
			showDisconnectFialDialog(res);
		}
		
		logger.log(Level.INFO, "Disconnect Successed: " + rInfo.getName() + " (" + rInfo.getAddr() + ")");
		
		// ڑEؒf{^̃gO
		toggleConDisConMenu();
		
		return res;
	}
	
	/**
	 * ToolBar쐬
	 */
	private void createToolBar() {
		toolBar = new ToolBar(shell, SWT.NONE);
		
		// {bg
		searchItem = new ToolItem(toolBar, SWT.NONE);
		searchItem.setImage(searchIcon);
		searchItem.setToolTipText(GUIDef.TOOLBAR_SEARCH_TIP[language]);
		searchItem.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Toolbar Search selected");
				searchRobot();
			}
		});
		
		// {bgo^
		registerItem = new ToolItem(toolBar, SWT.NONE);
		registerItem.setImage(registerIcon);
		registerItem.setToolTipText(GUIDef.TOOLBAR_REGISTER_TIP[language]);
		registerItem.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Toolbar Register selected");
				registerNewRobot();
			}
		});
		
		// {bg폜
		deleteItem = new ToolItem(toolBar, SWT.NONE);
		deleteItem.setImage(deleteIcon);
		deleteItem.setToolTipText(GUIDef.TOOLBAR_DEL_TIP[language]);
		deleteItem.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Toolbar Delete selected");
				deleteRobot(rInfoCurrent);
			}
		});
		
		// {bgƐڑ
		connectItem = new ToolItem(toolBar, SWT.NONE);
		connectItem.setImage(connectIcon);
		connectItem.setToolTipText(GUIDef.TOOLBAR_CONN_TIP[language]);
		connectItem.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Toolbar Connect selected");
				connectRobot(rInfoCurrent);
			}
		});
		
		// {bgƐؒf
		disconnectItem = new ToolItem(toolBar, SWT.NONE);
		disconnectItem.setImage(disconnectIcon);
		disconnectItem.setToolTipText(GUIDef.TOOLBAR_DISCONN_TIP[language]);
		disconnectItem.setEnabled(false);
		disconnectItem.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Toolbar Disconnect selected");
				disconnectRobot(rInfoCurrent);
			}
		});
		
		// Roboministratoȑ
		infoItem = new ToolItem(toolBar, SWT.NONE);
		infoItem.setImage(infoIcon);
		infoItem.setToolTipText(GUIDef.TOOLBAR_INFO_TIP[language]);
		infoItem.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				logger.log(Level.CONFIG, "Toolbar Help selected");
				showInfoDialog();
			}
		});
	}
	
	/**
	 * _CAO\(ǉs)
	 * @param errorCode \G[R[hw
	 */
	private void showAddFailDialog(int errorCode) {
		showErrorDialog(GUIDef.DIALOG_ERR_TITLE_ADD[language],
				GUIDef.DIALOG_ERR_MSG_ADD[language] + "\n" + 
				"ErrorCode (" + errorCode + ")");
	}
	
	/**
	 * _CAO\(ڑs)
	 * @param errorCode \G[R[hw
	 */
	private void showConnectFailDialog(int errorCode) {
		showErrorDialog(GUIDef.DIALOG_ERR_TITLE_CONN[language],
				GUIDef.DIALOG_ERR_MSG_CONN[language] + "\n" + 
				"ErrorCode (" + errorCode + ")");
	}
	
	/**
	 * _CAO\(ؒfs)
	 * @param errorCode \G[R[hw
	 */
	private void showDisconnectFialDialog(int errorCode) {
		showErrorDialog(GUIDef.DIALOG_ERR_TITLE_DISCONN[language],
				GUIDef.DIALOG_ERR_MSG_DISCONN[language] + "\n" + 
				"ErrorCode (" + errorCode + ")");
	}
	
	/**
	 * G[_CAO\
	 * @param title _CAÕ^Cg
	 * @param errMsg G[bZ[W
	 */
	private void showErrorDialog(String title, String errMsg) {
		MessageDialog dialog = new MessageDialog(
		        shell, title,
		        appliIcon,
		        errMsg,
		        MessageDialog.ERROR, new String[]{"OK"}, 1);
		dialog.open();
	}
	
	/**
	 * Roboministrator_CAO\
	 */
	private void showInfoDialog() {
		MessageDialog dialog = new MessageDialog(
		        shell, GUIDef.DIALOG_ABOUT_TITLE[language],
		        appliIcon,
		        GUIDef.DIALOG_ABOUT_MSG,
		        MessageDialog.INFORMATION, new String[]{"OK"}, 1);
		
		dialog.open();
	}
	
	/**
	 * \ύX̃_CAO\
	 */
	private void showLangDialog() {
		MessageDialog dialog = new MessageDialog(
		        shell, GUIDef.DIALOG_SET_LANG[language],
		        appliIcon,
		        GUIDef.DIALOG_SET_LANG_MSG[language],
		        MessageDialog.INFORMATION, new String[]{"OK"}, 1);
		
		dialog.open();	
	}
	
	/**
	 * {bgo^_CAO
	 * @author T.Kawashima
	 * @version 1.0.0
	 */
	class RegisterDialog extends Dialog {
		Label nameLabel;
		Text nameText;
		Label ipAddrLabel;
		Text ipAddrText;
		String name;
		String ipAddr;

		protected RegisterDialog(Shell parentShell) {
			super(parentShell);
		}
		
		protected Point getInitialSize() {
			return new Point(350, 250);
		}
		
		protected void configureShell(Shell newShell) {
			super.configureShell(newShell);
			newShell.setText(GUIDef.DIALOG_REG_TITLE[language]);
			newShell.setImage(appliIcon);
		}
		
		public String getNameValue() {
			return this.name;
		}
		
		public String getIpAddrValue() {
			return this.ipAddr;
		}

		protected Control createDialogArea(Composite parent) {
			// CAEgE}l[Wgpꍇ̓X[p[NX̃\bhĂ
			Composite composite = (Composite) super.createDialogArea(parent);
			composite.setLayout(new GridLayout(1, true));

			nameLabel = new Label(composite, SWT.NONE);
			nameLabel.setText(GUIDef.DIALOG_REG_NAME[language]);
			nameLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			nameText = new Text(composite, SWT.SINGLE | SWT.BORDER);
			nameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			ipAddrLabel = new Label(composite, SWT.NONE);
			ipAddrLabel.setText(GUIDef.DIALOG_REG_ADDR[language]);
			ipAddrLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			ipAddrText = new Text(composite, SWT.SINGLE | SWT.BORDER);
			ipAddrText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

			return composite;
		}
		
		protected void buttonPressed(int buttonId) {
			name = nameText.getText();
			ipAddr = ipAddrText.getText();
			super.buttonPressed(buttonId);
			close();
		}

	}
	
	/**
	 * o^_CAO\
	 * 
	 * @return InputDialog ̃\bh open() ̕Ԃl
	 */
	private int showRegisterDialog() {
		int res = 0;

		RegisterDialog dlg = new RegisterDialog(shell);
		res = dlg.open();
		
		if (res == Dialog.OK) {
			rInfoTemp = new RobotInfo(info, dlg.getNameValue(), dlg.getIpAddrValue());
			logger.log(Level.INFO, "Try to register " + dlg.getNameValue() + " ("+ dlg.getIpAddrValue() + ")");
		}
		
		return res;
	}
	
	/**
	 * RobotInfoǉ
	 * @param rInfo ǉRobotInfo
	 * @return 0As-1
	 */
	public synchronized int addRobotInfo(RobotInfo rInfo) {
		int robotNum = info.getRegisteredRobotNum();
		
		if (robotNum < RobominDef.PROPNTNUM_MAX) {
			addTreeItemToRoot(rInfo.getName(), rInfo.getAddr());
			info.addRobotInfo(rInfo);
			return RobominDef.SUCCESS;
		}
		
		return RobominDef.FAIL_SIZE;
	}
	
	/**
	 * c[ɓo^ꂽlbg^T[ǉ
	 * @param name lbg^T[̖O
	 * @param ipAddr lbg^T[IPAhX
	 * @return 0As-1
	 */
	public int addTreeItemToRoot(String name, String ipAddr) {
		int registeredNum = info.getRegisteredRobotNum();
		if (registeredNum < RobominDef.PROPNTNUM_MAX) {
			TreeItem newItem = new TreeItem(root, SWT.NONE);
			newItem.setText(name + " (" + ipAddr + ")");
			return RobominDef.SUCCESS;
		}
		
		return RobominDef.FAIL_SIZE;
	}
	
	/**
	 * CSashForm쐬
	 */
	private void createMainSash() {
		mainSashForm = new SashForm(shell, SWT.HORIZONTAL);
		mainSashForm.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		createLeftArea(mainSashForm);
		createRightArea(mainSashForm);
		
		mainSashForm.setWeights(new int[] {35, 65});
	}
	
	/**
	 * EBhE̍쐬
	 * @param sashForm eSashForm
	 */
	private void createLeftArea(SashForm sashForm) {
		leftAreaSashForm = new SashForm(sashForm, SWT.VERTICAL);
		
		// pl
		infoGroup = new Group(leftAreaSashForm, SWT.SHADOW_NONE);
		infoGroup.setText(GUIDef.GROUP_INFO[language]);
		infoGroup.setLayout(new GridLayout(1, true));
		infoGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		infoLabel = new Label(infoGroup, SWT.NONE);
		infoLabel.setLayoutData(new GridData(GridData.FILL_BOTH));
		infoLabel.setAlignment(SWT.CENTER);
		infoFont = new Font(display, "lr SVbN", 15, SWT.BOLD);
		infoLabel.setFont(infoFont);
		infoLabel.setText(GUIDef.GROUP_INFO_DEFAULT[language]);
		
		// J摜\
		cameraGroup = new Group(leftAreaSashForm, SWT.SHADOW_NONE);
		cameraGroup.setText(GUIDef.GROUP_CAMERA[language]);
		cameraGroup.setLayout(new GridLayout(1, true));
		cameraGroup.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		cameraCanvas = new Canvas(cameraGroup, SWT.NO_BACKGROUND);
		cameraCanvas.setLayoutData(new GridData(GridData.FILL_BOTH));
		backgroundImg = new Image(display, 320, 250);
		
		cameraCanvas.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent event) {
				try {
					// LȃAhXݒ肳Ăꍇ
					if (rInfoCurrent != null) {
						cameraImg = info.getNetTansorCmd().getTansorCameraImage(
								rInfoCurrent.getAddr());
						//GC bufferedGC = new GC(cameraImg);
						event.gc.drawImage(cameraImg, 0, 0);
					}
					else {
						event.gc.drawImage(backgroundImg, 0, 0);
					}
				} catch (Exception e) {
					logger.log(Level.SEVERE, "Error", e);
				}
			}
		});

		tree = new Tree(leftAreaSashForm, SWT.MULTI | SWT.BORDER);
		tree.addListener(SWT.MouseDown, new Listener() {
			public void handleEvent(Event e) {
				TreeItem item = tree.getItem(new Point(e.x, e.y));
				if (item != null) {
					// c[̃[t(o^ς݃{bg)Iꂽꍇ
					if (!item.getText().equals(GUIDef.TREE_ROOT[language])) {
						int startIndex = item.getText().indexOf("(");
						int endIndex = item.getText().indexOf(")");
						String selectedAddr = item.getText().substring(startIndex+1, endIndex);
						String selectedName = item.getText().substring(0, startIndex-1);
						
						RobotInfo selectedRInfo = null;
						
						// IꂽTreeItem̃CfbNXl擾łȂۂ̂ŋ̍
						// ̕@ƖOƃAhX̃{bg͒ǉłȂ悤ɂȂƁB
						selectedRInfo =info.getRobotInfo(selectedName, selectedAddr);
							
						// vRobotInfo݂Ȃꍇ
						if (selectedRInfo == null) {
							logger.log(Level.SEVERE, "There is no RobotInfo " + selectedName + " (" + selectedAddr + ")");
							return;
						}
						
						// Ver1.0.0ł1ZbV̂ݐڑƂ
						if (rInfoCurrent != null) {
							if (rInfoCurrent.isConnected()) {
								rInfoCurrent.disconnectRobot();
							}
						}
						
						rInfoCurrent = selectedRInfo;
						logger.log(Level.CONFIG, "Selected " + selectedName + " (" + selectedAddr + ")");
						writeInfoLabel(rInfoCurrent);
						
						// ENbN̏ꍇ|bvAbvj[o
						// ẼNbN̔@ŝߎb菈
						if (e.button == 3) {
							popup_leafMenu.setVisible(true);
						}
					}
					// c[̃[gIꂽꍇ
					else if(item.getText().equals(GUIDef.TREE_ROOT[language])) {
						if (e.button == 3) {
							popup_rootMenu.setVisible(true);
						}
					}
				}
			}
		});
		
		root = new TreeItem(tree, SWT.NONE);
		root.setText(GUIDef.TREE_ROOT[language]);

		leftAreaSashForm.setWeights(new int[] {1, 5, 4});
	}
	
	/**
	 * EBhẺE쐬
	 * @param sashForm eSashForm
	 */
	private void createRightArea(SashForm sashForm) {
		tabFolder = new CTabFolder(sashForm, SWT.BORDER);
		tabFolder.setTabHeight(25);
		tabFolder.setSimple(false);

		Color titleForeColor =
		display.getSystemColor(SWT.COLOR_TITLE_FOREGROUND);
		Color titleBackColor1 =
		display.getSystemColor(SWT.COLOR_TITLE_BACKGROUND);
		Color titleBackColor2 =
		display.getSystemColor(SWT.COLOR_TITLE_BACKGROUND_GRADIENT);

		// IĂ^u̕F
		tabFolder.setSelectionForeground(titleForeColor);
		
		// IĂ^u̔wi
		tabFolder.setSelectionBackground(
		new Color[] {titleBackColor1, titleBackColor2},
		new int[] {100}, true);
		
		nettansorTab = new NetTansorTab(tabFolder, info);
		mindstormTab = new MINDSTORMTab(tabFolder, info);
		consoleTab = new ConsoleTab(tabFolder, info);
	}
	
	/**
	 * FPS(J)ύX_CAO
	 * @author T.Kawashima
	 * @version 1.0.0
	 */
	class FpsChangeDialog extends Dialog {
		Scale scale;
		Text text;

		protected FpsChangeDialog(Shell parentShell) {
			super(parentShell);
			
		}
		
		protected Point getInitialSize() {
			return new Point(250, 120);
		}
		
		protected void configureShell(Shell newShell) {
			super.configureShell(newShell);
			newShell.setText(GUIDef.DIALOG_SET_FPS[language]);
			newShell.setImage(appliIcon);
		}

		protected Control createDialogArea(Composite parent) {
			// CAEgE}l[Wgpꍇ̓X[p[NX̃\bhĂ
			Composite composite = (Composite) super.createDialogArea(parent);
			composite.setLayout(new GridLayout(2, false));

			scale = new Scale(composite, SWT.HORIZONTAL);
			scale.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			scale.setMinimum(RobominDef.CAMERA_FPS_MIN);
			scale.setMaximum(RobominDef.CAMERA_FPS_MAX);
			scale.setIncrement(GUIDef.DIALOG_FPS_INCREMENT);
			scale.setPageIncrement(GUIDef.DIALOG_FPS_PINCREMENT);
			scale.setSelection(cameraFps);
			text = new Text(composite, SWT.BORDER);
			text.setEditable(false);
			text.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			text.setText(String.valueOf(scale.getSelection()));

			scale.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					int value = scale.getSelection();
					text.setText(String.valueOf(value));
					cameraFpsTemp = value;
				}
			});

			return composite;
		}
	}
	
	/**
	 * FPS(J)ݒ肷
	 * @param fps ݒ肷FPS
	 * @return 0As-1
	 */
	
	public int setCameraFps(int fps) {
		if (fps >= RobominDef.CAMERA_FPS_MIN && fps <= RobominDef.CAMERA_FPS_MAX) {
			cameraFps = fps;
			redrawTime = 1000 / cameraFps;
			return RobominDef.SUCCESS;
		}
		else {
			return RobominDef.FAIL_SIZE;
		}
	}
	
	/**
	 * JXbh̗LԂ
	 * @return threadFlag
	 */
	public boolean isCameraThread() {
		return this.threadFlag;
	}
	
	/**
	 * J摜\CanvasXV(redraw)
	 */
	public void cameraThreadStart() {
		threadFlag = true;
		Thread cameraThread = new Thread() {
			public void run() {
				while (threadFlag) {
					try {
						display.asyncExec(new Runnable() {
							public void run() {
								cameraCanvas.redraw();
							}
						});
						Thread.sleep(redrawTime);
					}
					catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
		};
		cameraThread.start();
		logger.log(Level.INFO, "Camera started");
	}
	
	/**
	 * J摜̍XV~
	 */
	public void cameraThreadStop() {
		cameraThreadStop(true);
	}
	
	/**
	 * J摜̍XV~
	 * @param logFlag Oo̗͂L
	 */
	public void cameraThreadStop(boolean logFlag) {
		threadFlag = false;
		if (logFlag) logger.log(Level.INFO, "Camera stopped");
	}
	
	/**
	 * ShellԂ
	 * @return shell
	 */
	public Shell getShell() {
		return this.shell;
	}
	
	/**
	 * SWTp[v
	 */
	public void guiThreadLoop() {
		shell.open();
		
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		display.dispose();
	}
}
